
/*!
 *  @brief Subresultant mapping matrix w.r.t. f and g of order r (Sylvester matrix is r=0).
 */
void SNAP_agcd_gpgcd_subresultant_mapping_matrix_dsc_mix(SNAPMTXREAL_t matrix, const unsigned int x, const unsigned int y, SNAPMTXREAL_t vecD, const unsigned int m, const unsigned int n, const unsigned int d) {
  unsigned int i, j, r;
  SNAPBASEREAL_t *pt1, *pt2;
#if defined(SNAP_SUPDC_T) || defined(SNAP_SUPFC_T)
  unsigned int hcols, hrows, fcols, gcols;
  SNAPBASEREAL_t *pt3, *pt4, *pt5, *pt6;
#endif
  // the matrix size will be rows x cols.
  r = d+1;
#if defined(SNAP_SUPDC_T) || defined(SNAP_SUPFC_T)
  fcols = m+1;
  gcols = n+1;
  hcols = fcols + gcols - 2*r - 2;
  hrows = hcols + r;
#endif
#if defined(SNAP_SUPD_T) || defined(SNAP_SUPFR_T)
  // poly_f
  for ( i=0; i<n-r; i++ ) {
    for ( j=0, pt1=SNAPMTXREAL_elemp(matrix,x+i,y+i), pt2=SNAPMTXREAL_elems(vecD); j<=m; j++,pt1+=SNAPMTXREAL_csize(matrix),pt2++ ) {
      SNAPBASEREAL_set(*pt1, *pt2);
    }
  }
  // poly_g
  for ( i=0; i<m-r; i++ ) {
    for ( j=0, pt1=SNAPMTXREAL_elemp(matrix,x+i,y+i+n-r), pt2=SNAPMTXREAL_elemp(vecD,0,m+1); j<=n; j++,pt1+=SNAPMTXREAL_csize(matrix),pt2++ ) {
      SNAPBASEREAL_set(*pt1, *pt2);
    }
  }
#else
  // poly_f
  for ( i=0; i<gcols-r-1; i++ ) {
    for ( j=0, pt1=SNAPMTXREAL_elemp(matrix,x+i,y+i), pt2=SNAPMTXREAL_elemp(matrix,x+i+hrows,y+i), pt3=SNAPMTXREAL_elemp(matrix,x+i,y+i+hcols), pt4=SNAPMTXREAL_elemp(matrix,x+i+hrows,y+i+hcols), pt5=SNAPMTXREAL_elems(vecD), pt6=SNAPMTXREAL_elemp(vecD,0,m+n+2);
          j<fcols;
          j++,pt1+=SNAPMTXREAL_csize(matrix),pt2+=SNAPMTXREAL_csize(matrix),pt3+=SNAPMTXREAL_csize(matrix),pt4+=SNAPMTXREAL_csize(matrix),pt5++,pt6++ ) {
      SNAPBASEREAL_set(*pt1, *pt5);
      SNAPBASEREAL_set(*pt4, *pt5);
      SNAPBASEREAL_set(*pt2, *pt6);
      SNAPBASEREAL_neg(*pt3, *pt6);
    }
  }
  // poly_g
  for ( i=0; i<fcols-r-1; i++ ) {
    for ( j=0, pt1=SNAPMTXREAL_elemp(matrix,x+i,y+i+gcols-r-1), pt2=SNAPMTXREAL_elemp(matrix,x+i+hrows,y+i+gcols-r-1), pt3=SNAPMTXREAL_elemp(matrix,x+i,y+i+hcols+gcols-r-1), pt4=SNAPMTXREAL_elemp(matrix,x+i+hrows,y+i+hcols+gcols-r-1), pt5=SNAPMTXREAL_elemp(vecD,0,m+1), pt6=SNAPMTXREAL_elemp(vecD,0,2*m+n+3);
          j<gcols;
          j++,pt1+=SNAPMTXREAL_csize(matrix),pt2+=SNAPMTXREAL_csize(matrix),pt3+=SNAPMTXREAL_csize(matrix),pt4+=SNAPMTXREAL_csize(matrix),pt5++,pt6++ ) {
      SNAPBASEREAL_set(*pt1, *pt5);
      SNAPBASEREAL_set(*pt4, *pt5);
      SNAPBASEREAL_set(*pt2, *pt6);
      SNAPBASEREAL_neg(*pt3, *pt6);
    }
  }
#endif
}

/*!
 *  @brief line search in GPGCD algorithm for two univariate polynomials.
 *  @return the step size calculated.
 */
SNAPBASEREAL_t SNAP_agcd_gpgcd_linesearch(SNAPMTXREAL_t input_f, SNAPMTXREAL_t input_g, SNAPMTXREAL_t matJ, SNAPMTXREAL_t matT, SNAPMTXREAL_t vecD, SNAPMTXREAL_t vecT, const unsigned int m, const unsigned int n, const unsigned int d) {
  unsigned int i, num_iterations;
  SNAPBASEREAL_t *pt1, *pt2;
  SNAPBASEREAL_t tmp1, tmp2, alpha;
  SNAPBASEREAL_init(tmp1);
  SNAPBASEREAL_init(tmp2);
  SNAPBASEREAL_init(alpha);

  // f2: f0+f1*a+f2*a^2
  SNAPMTXREAL_BLAS_dot(SNAPMTXREAL_elem(matT,0,2), SNAPMTXREAL_cols(input_f)+SNAPMTXREAL_cols(input_g), SNAPMTXREAL_elems(vecD), 1, SNAPMTXREAL_elems(vecD), 1);
  // f1: f0+f1*a+f2*a^2
  SNAPMTXREAL_BLAS_dot(SNAPMTXREAL_elem(matT,0,1), m+1, SNAPMTXREAL_elemp(matJ,1,m+n+2), SNAPMTXREAL_csize(matJ), SNAPMTXREAL_elems(vecD), 1);
  SNAPMTXREAL_BLAS_dot(tmp1, n+1, SNAPMTXREAL_elemp(matJ,1,m+2*n-d+3), SNAPMTXREAL_csize(matJ), SNAPMTXREAL_elems(vecD)+m+1, 1);
  SNAPBASEREAL_add(SNAPMTXREAL_elem(matT,0,1), SNAPMTXREAL_elem(matT,0,1), tmp1);
  SNAPMTXREAL_BLAS_dot(tmp1, m+1, SNAPMTXREAL_elems(input_f), 1, SNAPMTXREAL_elems(vecD), 1);
  SNAPBASEREAL_sub(SNAPMTXREAL_elem(matT,0,1), SNAPMTXREAL_elem(matT,0,1), tmp1);
  SNAPMTXREAL_BLAS_dot(tmp1, n+1, SNAPMTXREAL_elems(input_g), 1, SNAPMTXREAL_elems(vecD)+m+1, 1);
  SNAPBASEREAL_sub(SNAPMTXREAL_elem(matT,0,1), SNAPMTXREAL_elem(matT,0,1), tmp1);
  SNAPBASEREAL_mul_ui(SNAPMTXREAL_elem(matT,0,1), SNAPMTXREAL_elem(matT,0,1), 2);
  // f0: f0+f1*a+f2*a^2
  SNAPMTXREAL_BLAS_dot(SNAPMTXREAL_elem(matT,0,0), m+1, SNAPMTXREAL_elems(input_f), 1, SNAPMTXREAL_elems(input_f), 1);
  SNAPMTXREAL_BLAS_dot(tmp1, n+1, SNAPMTXREAL_elems(input_g), 1, SNAPMTXREAL_elems(input_g), 1);
  SNAPBASEREAL_add(SNAPMTXREAL_elem(matT,0,0), SNAPMTXREAL_elem(matT,0,0), tmp1);
  SNAPMTXREAL_BLAS_dot(tmp1, m+1, SNAPMTXREAL_elemp(matJ,1,m+n+2), SNAPMTXREAL_csize(matJ), SNAPMTXREAL_elemp(matJ,1,m+n+2), SNAPMTXREAL_csize(matJ));
  SNAPBASEREAL_add(SNAPMTXREAL_elem(matT,0,0), SNAPMTXREAL_elem(matT,0,0), tmp1);
  SNAPMTXREAL_BLAS_dot(tmp1, n+1, SNAPMTXREAL_elemp(matJ,1,m+2*n-d+3), SNAPMTXREAL_csize(matJ), SNAPMTXREAL_elemp(matJ,1,m+2*n-d+3), SNAPMTXREAL_csize(matJ));
  SNAPBASEREAL_add(SNAPMTXREAL_elem(matT,0,0), SNAPMTXREAL_elem(matT,0,0), tmp1);
  SNAPMTXREAL_BLAS_dot(tmp1, m+1, SNAPMTXREAL_elems(input_f), 1, SNAPMTXREAL_elemp(matJ,1,m+n+2), SNAPMTXREAL_csize(matJ));
  SNAPMTXREAL_BLAS_dot(tmp2, n+1, SNAPMTXREAL_elems(input_g), 1, SNAPMTXREAL_elemp(matJ,1,m+2*n-d+3), SNAPMTXREAL_csize(matJ));
  SNAPBASEREAL_add(tmp1, tmp1, tmp2);
  SNAPBASEREAL_mul_ui(tmp1, tmp1, 2);
  SNAPBASEREAL_sub(SNAPMTXREAL_elem(matT,0,0), SNAPMTXREAL_elem(matT,0,0), tmp1);
  // 2v -> v
  for ( i=0,pt1=SNAPMTXREAL_elemp(matJ,0,m+n+2); i<m+n-2*d+2; i++,pt1++ ) {
    SNAPBASEREAL_div_ui(*pt1, *pt1, 2);
  }
  // 0: N.v: (N+aN).(v+av)
  cblas_dgemv(CblasRowMajor, CblasNoTrans, m+n-d+2, m+n-2*d+2, 1.0, SNAPMTXREAL_elemp(matJ,0,m+n+2), SNAPMTXREAL_csize(matJ),
              SNAPMTXREAL_elemp(matJ,0,m+n+2), 1, 0.0, SNAPMTXREAL_elemp(matT,1,0), SNAPMTXREAL_csize(matT));
  SNAPBASEREAL_sub_ui(SNAPMTXREAL_elem(matT,1,0), SNAPMTXREAL_elem(matT,1,0), 1);
  // 1: N.av (and av.v): (N+aN).(v+av)
  cblas_dgemv(CblasRowMajor, CblasNoTrans, m+n-d+2, m+n-2*d+2, 1.0, SNAPMTXREAL_elemp(matJ,0,m+n+2), SNAPMTXREAL_csize(matJ),
              SNAPMTXREAL_elemp(vecD,0,m+n+2), 1, 0.0, SNAPMTXREAL_elemp(matT,1,1), SNAPMTXREAL_csize(matT));
  SNAPBASEREAL_mul_ui(SNAPMTXREAL_elem(matT,1,1), SNAPMTXREAL_elem(matT,1,1), 2);
  // update subresultant matrix
  SNAP_agcd_gpgcd_subresultant_mapping_matrix_dsc_mix(matJ, 1, m+n+2, vecD, m, n, d);
  // 1: aN.v (wo av.v): (N+aN).(v+av)
  cblas_dgemv(CblasRowMajor, CblasNoTrans, m+n-d+1, m+n-2*d+2, 1.0, SNAPMTXREAL_elemp(matJ,1,m+n+2), SNAPMTXREAL_csize(matJ),
              SNAPMTXREAL_elemp(matJ,0,m+n+2), 1, 1.0, SNAPMTXREAL_elemp(matT,2,1), SNAPMTXREAL_csize(matT));
  // update av
  for ( i=0,pt1=SNAPMTXREAL_elemp(matJ,0,m+n+2),pt2=SNAPMTXREAL_elemp(vecD,0,m+n+2); i<m+n-2*d+2; i++,pt1++,pt2++ ) {
    SNAPBASEREAL_set(*pt1, *pt2);
  }
  // 2: aN.av: (N+aN).(v+av)
  cblas_dgemv(CblasRowMajor, CblasNoTrans, m+n-d+2, m+n-2*d+2, 1.0, SNAPMTXREAL_elemp(matJ,0,m+n+2), SNAPMTXREAL_csize(matJ),
              SNAPMTXREAL_elemp(vecD,0,m+n+2), 1, 0.0, SNAPMTXREAL_elemp(matT,1,2), SNAPMTXREAL_csize(matT));
#define VEC_LEN_ m+n-d+3
#define VEC_LEN m+n-d+2
#define VEC_START_ 0
#define VEC_START 1
  // a0: a0+a1*a+a2*a^2+a3*a^3+a4*a^4
  SNAPMTXREAL_BLAS_dot(SNAPMTXREAL_elem(vecT,0,0), VEC_LEN, SNAPMTXREAL_elemp(matT,VEC_START,0), SNAPMTXREAL_csize(matT), SNAPMTXREAL_elemp(matT,VEC_START,0), SNAPMTXREAL_csize(matT));
  // a1: a0+a1*a+a2*a^2+a3*a^3+a4*a^4
  SNAPMTXREAL_BLAS_dot(SNAPMTXREAL_elem(vecT,0,1), VEC_LEN, SNAPMTXREAL_elemp(matT,VEC_START,0), SNAPMTXREAL_csize(matT), SNAPMTXREAL_elemp(matT,VEC_START,1), SNAPMTXREAL_csize(matT));
  SNAPBASEREAL_mul_ui(SNAPMTXREAL_elem(vecT,0,1), SNAPMTXREAL_elem(vecT,0,1), 2);
  // a2: a0+a1*a+a2*a^2+a3*a^3+a4*a^4
  SNAPMTXREAL_BLAS_dot(SNAPMTXREAL_elem(vecT,0,2), VEC_LEN, SNAPMTXREAL_elemp(matT,VEC_START,1), SNAPMTXREAL_csize(matT), SNAPMTXREAL_elemp(matT,VEC_START,1), SNAPMTXREAL_csize(matT));
  SNAPMTXREAL_BLAS_dot(tmp1, VEC_LEN, SNAPMTXREAL_elemp(matT,VEC_START,0), SNAPMTXREAL_csize(matT), SNAPMTXREAL_elemp(matT,VEC_START,2), SNAPMTXREAL_csize(matT));
  SNAPBASEREAL_mul_ui(tmp1, tmp1, 2);
  SNAPBASEREAL_add(SNAPMTXREAL_elem(vecT,0,2), SNAPMTXREAL_elem(vecT,0,2), tmp1);
  // a3: a0+a1*a+a2*a^2+a3*a^3+a4*a^4
  SNAPMTXREAL_BLAS_dot(SNAPMTXREAL_elem(vecT,0,3), VEC_LEN, SNAPMTXREAL_elemp(matT,VEC_START,1), SNAPMTXREAL_csize(matT), SNAPMTXREAL_elemp(matT,VEC_START,2), SNAPMTXREAL_csize(matT));
  SNAPBASEREAL_mul_ui(SNAPMTXREAL_elem(vecT,0,3), SNAPMTXREAL_elem(vecT,0,3), 2);
  // a4: a0+a1*a+a2*a^2+a3*a^3+a4*a^4
  SNAPMTXREAL_BLAS_dot(SNAPMTXREAL_elem(vecT,0,4), VEC_LEN, SNAPMTXREAL_elemp(matT,VEC_START,2), SNAPMTXREAL_csize(matT), SNAPMTXREAL_elemp(matT,VEC_START,2), SNAPMTXREAL_csize(matT));

printf("coef: %e %e %e %e %e\n", SNAPMTXREAL_elem(vecT,0,0),SNAPMTXREAL_elem(vecT,0,1),SNAPMTXREAL_elem(vecT,0,2),SNAPMTXREAL_elem(vecT,0,3),SNAPMTXREAL_elem(vecT,0,4));
  // newton-method
  num_iterations = 0;
  SNAPBASEREAL_set_d(alpha, 1.0);
  while (1) {
    // a'(alpha)
    SNAPBASEREAL_mul(tmp2, alpha, SNAPMTXREAL_elem(vecT,0,4));
    SNAPBASEREAL_mul_ui(tmp2, tmp2, 4);
    SNAPBASEREAL_mul_ui(tmp1, SNAPMTXREAL_elem(vecT,0,3), 3);
    SNAPBASEREAL_add(tmp2, tmp2, tmp1);
    SNAPBASEREAL_mul(tmp2, tmp2, alpha);
    SNAPBASEREAL_mul_ui(tmp1, SNAPMTXREAL_elem(vecT,0,2), 2);
    SNAPBASEREAL_add(tmp2, tmp2, tmp1);
    SNAPBASEREAL_mul(tmp2, tmp2, alpha);
    SNAPBASEREAL_add(tmp2, tmp2, SNAPMTXREAL_elem(vecT,0,1));
    // a(alpha)
    SNAPBASEREAL_mul(tmp1, alpha, SNAPMTXREAL_elem(vecT,0,4));
    SNAPBASEREAL_add(tmp1, tmp1, SNAPMTXREAL_elem(vecT,0,3));
    SNAPBASEREAL_mul(tmp1, tmp1, alpha);
    SNAPBASEREAL_add(tmp1, tmp1, SNAPMTXREAL_elem(vecT,0,2));
    SNAPBASEREAL_mul(tmp1, tmp1, alpha);
    SNAPBASEREAL_add(tmp1, tmp1, SNAPMTXREAL_elem(vecT,0,1));
    SNAPBASEREAL_mul(tmp1, tmp1, alpha);
    SNAPBASEREAL_add(tmp1, tmp1, SNAPMTXREAL_elem(vecT,0,0));
    // correction term
    SNAPBASEREAL_div(tmp1, tmp1, tmp2);
    if ( SNAPBASEREAL_cmpabs(tmp1, SNAPLIB_GPGCD_THRESHOLD) < 0 ) {
      SNAPBASEREAL_sub(alpha, alpha, tmp1);
      break;
    }
    SNAPBASEREAL_sub(alpha, alpha, tmp1);
    if ( ++num_iterations >= 3 ) {
      break;
    }
  }
  SNAPBASEREAL_clear(tmp1);
  SNAPBASEREAL_clear(tmp2);
  SNAPBASEREAL_clear(alpha);

  return alpha;
}

