/*
 * Copyright (c) 2012-2015, Kosaku Nagasaka <nagasaka@main.h.kobe-u.ac.jp>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <snap.h>

// #define SNAPLIB_DEBUG
#undef SNAPLIB_DEBUG

void sp_lexical_token_init(sp_lexical_token_t token) {
  token->token = NULL;
  token->next = NULL;
}

void sp_lexical_token_clear(sp_lexical_token_t token) {
  if ( token->token != NULL ) {
    free(token->token);
    token->token = NULL;
  }
  token->next = NULL;
}

void sp_lexical_token_free(sp_lexical_token_t head) {
  sp_lexical_token_struct *current, *tobefree;
  if ( head == NULL ) {
    return;
  }
  current = head->next;
  while ( current != NULL ) {
    tobefree = current;
    current = current->next;
    sp_lexical_token_clear(tobefree);
    free(tobefree);
  }
  sp_lexical_token_clear(head);
}

#define SNAPLIB_TRUE 1
#define SNAPLIB_FALSE 0

int sp_lexical_analysis_str(sp_lexical_token_t head, char *buf) {
  sp_lexical_token_struct *current, *new;
  unsigned long long int len;
  short int dotq, powerq, zerostartq, errorq, dotstartq;
  current = head;
  while ( *buf != '\0' ) {
    if ( (9 <= *buf && *buf <= 13) || *buf == 32 ) { // WHITE SPACE
#ifdef SNAPLIB_DEBUG
      printf("WHITE SPACE\n");
#endif
      // increment string buffer
      buf++;
    } else if ( *buf == '(' ) { // SNAP_SP_LPAR
#ifdef SNAPLIB_DEBUG
      printf("SNAP_SP_LPAR\n");
#endif
      current->type = SNAP_SP_LPAR;
      current->token = (char*)malloc(sizeof(char)*(1+1));
      strncpy(current->token, "(", 2);
      // increment string buffer
      buf++;
      // create new token space
      new = (sp_lexical_token_struct*) malloc(sizeof(sp_lexical_token_struct));
      sp_lexical_token_init(new);
      current->next = new;
      current = new;
    } else if ( *buf == ')' ) { // SNAP_SP_RPAR
#ifdef SNAPLIB_DEBUG
      printf("SNAP_SP_RPAR\n");
#endif
      current->type = SNAP_SP_RPAR;
      current->token = (char*)malloc(sizeof(char)*(1+1));
      strncpy(current->token, ")", 2);
      // increment string buffer
      buf++;
      // create new token space
      new = (sp_lexical_token_struct*) malloc(sizeof(sp_lexical_token_struct));
      sp_lexical_token_init(new);
      current->next = new;
      current = new;
    } else if ( *buf == '+' ) { // SNAP_SP_ADD
#ifdef SNAPLIB_DEBUG
      printf("SNAP_SP_ADD\n");
#endif
      current->type = SNAP_SP_ADD;
      current->token = (char*)malloc(sizeof(char)*(1+1));
      strncpy(current->token, "+", 2);
      // increment string buffer
      buf++;
      // create new token space
      new = (sp_lexical_token_struct*) malloc(sizeof(sp_lexical_token_struct));
      sp_lexical_token_init(new);
      current->next = new;
      current = new;
    } else if ( *buf == '-' ) { // SNAP_SP_SUB
#ifdef SNAPLIB_DEBUG
      printf("SNAP_SP_SUB\n");
#endif
      current->type = SNAP_SP_SUB;
      current->token = (char*)malloc(sizeof(char)*(1+1));
      strncpy(current->token, "-", 2);
      // increment string buffer
      buf++;
      // create new token space
      new = (sp_lexical_token_struct*) malloc(sizeof(sp_lexical_token_struct));
      sp_lexical_token_init(new);
      current->next = new;
      current = new;
    } else if ( *buf == '*' ) { // SNAP_SP_MUL
#ifdef SNAPLIB_DEBUG
      printf("SNAP_SP_MUL\n");
#endif
      current->type = SNAP_SP_MUL;
      current->token = (char*)malloc(sizeof(char)*(1+1));
      strncpy(current->token, "*", 2);
      // increment string buffer
      buf++;
      // create new token space
      new = (sp_lexical_token_struct*) malloc(sizeof(sp_lexical_token_struct));
      sp_lexical_token_init(new);
      current->next = new;
      current = new;
    } else if ( *buf == '^' ) { // SNAP_SP_POW
#ifdef SNAPLIB_DEBUG
      printf("SNAP_SP_POW\n");
#endif
      current->type = SNAP_SP_POW;
      current->token = (char*)malloc(sizeof(char)*(1+1));
      strncpy(current->token, "^", 2);
      // increment string buffer
      buf++;
      // create new token space
      new = (sp_lexical_token_struct*) malloc(sizeof(sp_lexical_token_struct));
      sp_lexical_token_init(new);
      current->next = new;
      current = new;
    } else if ( (*buf == 'I' || *buf == 'i') && (*(buf+1) == '\0' || !(('A' <= *(buf+1) && *(buf+1) <= 'Z') || ('a' <= *(buf+1) && *(buf+1) <= 'z') || ('0' <= *(buf+1) && *(buf+1) <= '9') || *(buf+1) == '_') ) ) {
      // SNAP_SP_J (imaginary unit I)
#ifdef SNAPLIB_DEBUG
      printf("SNAP_SP_J\n");
#endif
      current->type = SNAP_SP_J;
      current->token = (char*)malloc(sizeof(char)*(1+1));
      strncpy(current->token, "I", 2);
      // increment string buffer
      buf++;
      // create new token space
      new = (sp_lexical_token_struct*) malloc(sizeof(sp_lexical_token_struct));
      sp_lexical_token_init(new);
      current->next = new;
      current = new;
    } else if ( ('A' <= *buf && *buf <= 'Z') || ('a' <= *buf && *buf <= 'z') ) {
      // SNAP_SP_ID (thought in general there are other possibilities)
#ifdef SNAPLIB_DEBUG
      printf("SNAP_SP_ID\n");
#endif
      for(len=1; ('A' <= *(buf+len) && *(buf+len) <= 'Z') ||
          ('a' <= *(buf+len) && *(buf+len) <= 'z') ||
          ('0' <= *(buf+len) && *(buf+len) <= '9') || *(buf+len) == '_'; len++ );
      current->type = SNAP_SP_ID;
      current->token = (char*)malloc(sizeof(char)*(len+1));
      strncpy(current->token, buf, len);
      current->token[len] = '\0';
      // increment string buffer
      buf = buf + len;
      // create new token space
      new = (sp_lexical_token_struct*) malloc(sizeof(sp_lexical_token_struct));
      sp_lexical_token_init(new);
      current->next = new;
      current = new;
    } else if ( ('0' <= *buf && *buf <= '9') || *buf == '.' ) {
      // SNAP_SP_INT or SNAP_SP_REAL
#ifdef SNAPLIB_DEBUG
      printf("SNAP_SP_INT or SNAP_SP_REAL\n");
#endif
      dotq = SNAPLIB_FALSE;
      powerq = SNAPLIB_FALSE;
      zerostartq = SNAPLIB_FALSE;
      errorq = SNAPLIB_FALSE;
      dotstartq = SNAPLIB_FALSE;
      if ( *buf == '0' ) {
        zerostartq = SNAPLIB_TRUE;
      }
      if ( *buf == '.' ) {
        dotq =SNAPLIB_TRUE;
        dotstartq = SNAPLIB_TRUE;
      }
      for ( len = 1; ('0' <= *(buf+len) && *(buf+len) <= '9') ||
            *(buf+len) == '.' || *(buf+len) == 'e' || *(buf+len) == 'E'; len++ ) {
        if ( zerostartq == SNAPLIB_TRUE ) {
          if ( *(buf+len) == 'e' || *(buf+len) == 'E' ) {
            zerostartq = SNAPLIB_FALSE;
          } else if ( *(buf+len) == '.' && dotstartq != SNAPLIB_TRUE ) {
            zerostartq = SNAPLIB_FALSE;
          } else {
            errorq = SNAPLIB_TRUE;
            break;
          }
        }
        if ( *(buf+len) == '.' ) {
          if ( dotq == SNAPLIB_FALSE && powerq == SNAPLIB_FALSE && dotstartq == SNAPLIB_FALSE ) {
            dotq = SNAPLIB_TRUE;
          } else {
            errorq = SNAPLIB_TRUE;
            break;
          }
        } else if ( *(buf+len) == 'e' || *(buf+len) == 'E' ) {
          if ( powerq == SNAPLIB_FALSE ) {
            powerq = SNAPLIB_TRUE;
            if ( *(buf+len+1) == '+' || *(buf+len+1) == '-' ) {
              len++;
            }
          } else {
            errorq = SNAPLIB_TRUE;
            break;
          }
        }
      }
      if ( errorq == SNAPLIB_TRUE ) { // ERROR
        sp_lexical_token_free(head);
        return -1;
      }
      if ( dotq == SNAPLIB_TRUE || powerq == SNAPLIB_TRUE ) {
        current->type = SNAP_SP_REAL;
      } else {
        current->type = SNAP_SP_INT;
      }
      if ( dotstartq == SNAPLIB_TRUE ) {
        current->token = (char*)malloc(sizeof(char)*(len+2));
        *(current->token) = '0';
        strncpy(current->token+1, buf, len);
        current->token[len+1] = '\0';
      } else {
        current->token = (char*)malloc(sizeof(char)*(len+1));
        strncpy(current->token, buf, len);
        current->token[len] = '\0';
      }
      // increment string buffer
      buf = buf + len;
      // create new token space
      new = (sp_lexical_token_struct*) malloc(sizeof(sp_lexical_token_struct));
      sp_lexical_token_init(new);
      current->next = new;
      current = new;
    } else {
      // ERROR
#ifdef SNAPLIB_DEBUG
      printf("ERROR\n");
#endif
      sp_lexical_token_free(head);
      return -1;
    }
  }
  current->type = SNAP_SP_EOF;
#ifdef SNAPLIB_DEBUG
  printf("SNAP_SP_EOF\n");
#endif
  return 0;
}

