/*
 * Copyright (c) 2013-2015, Kosaku Nagasaka <nagasaka@main.h.kobe-u.ac.jp>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <stdlib.h>
#include <snap.h>

void mpc_init_set(mpc_t rp, mpc_t op, mpc_rnd_t mode) {
  mpc_init2(rp, mpfr_get_default_prec());
  mpc_set(rp, op, mode);
}

void mpc_init_set_ui(mpc_t rp, unsigned int op, mpc_rnd_t mode) {
  mpc_init2(rp, mpfr_get_default_prec());
  mpc_set_ui(rp, op, mode);
}

void mpc_init_set_si(mpc_t rp, signed int op, mpc_rnd_t mode) {
  mpc_init2(rp, mpfr_get_default_prec());
  mpc_set_si(rp, op, mode);
}

void mpc_sub_d(mpc_t rp, mpc_t op1, double op2, mpc_rnd_t mode) {
  mpfr_sub_d(mpc_realref(rp), mpc_realref(op1), op2, MPC_RND_RE(mode));
  mpfr_set(mpc_imagref(rp), mpc_imagref(op1), MPC_RND_IM(mode));
}

void mpc_sub_dc(mpc_t rp, mpc_t op1, dcomplex op2, mpc_rnd_t mode) {
  mpfr_sub_d(mpc_realref(rp), mpc_realref(op1), creal(op2), MPC_RND_RE(mode));
  mpfr_sub_d(mpc_imagref(rp), mpc_imagref(op1), cimag(op2), MPC_RND_IM(mode));
}

void mpc_mul_d(mpc_t rp, mpc_t op1, double op2, mpc_rnd_t mode) {
  mpfr_mul_d(mpc_realref(rp), mpc_realref(op1), op2, MPC_RND_RE(mode));
  mpfr_mul_d(mpc_imagref(rp), mpc_imagref(op1), op2, MPC_RND_IM(mode));
}

void mpc_mul_dc(mpc_t rp, mpc_t op1, dcomplex op2, mpc_rnd_t mode) {
  mpc_t mpc_op2;
  mpc_init2(mpc_op2, mpfr_get_default_prec());
  mpc_set_dc(mpc_op2, op2, mode);
  mpc_mul(rp, op1, mpc_op2, mode);
  mpc_clear(mpc_op2);
}

void mpc_div_d(mpc_t rp, mpc_t op1, double op2, mpc_rnd_t mode) {
  mpfr_div_d(mpc_realref(rp), mpc_realref(op1), op2, MPC_RND_RE(mode));
  mpfr_div_d(mpc_imagref(rp), mpc_imagref(op1), op2, MPC_RND_IM(mode));
}

void mpc_div_dc(mpc_t rp, mpc_t op1, dcomplex op2, mpc_rnd_t mode) {
  mpc_t mpc_op2;
  mpc_init2(mpc_op2, mpfr_get_default_prec());
  mpc_set_dc(mpc_op2, op2, mode);
  mpc_div(rp, op1, mpc_op2, mode);
  mpc_clear(mpc_op2);
}















