/*
 * Copyright (c) 2012-2015, Kosaku Nagasaka <nagasaka@main.h.kobe-u.ac.jp>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <stdlib.h>
#include <snap.h>

void mpq_sub_z(mpq_t x, mpq_t y1, mpz_t y2) {
  mpq_t tmp;
  mpq_init(tmp);
  mpq_set_z(tmp, y2);
  mpq_sub(x,y1,tmp);
  mpq_clear(tmp);
}

void mpq_mul_z(mpq_t x, mpq_t y1, mpz_t y2) {
  mpq_t tmp;
  mpq_init(tmp);
  mpq_set_z(tmp, y2);
  mpq_mul(x,y1,tmp);
  mpq_clear(tmp);
}

void mpq_get_z(mpz_t x, mpq_t y) {
  mpz_t r;
  mpz_init(r);
  mpz_tdiv_qr(x, r, mpq_numref(y), mpq_denref(y));
  mpz_mul_ui(r,r,2);
  if (mpz_cmpabs(r,mpq_denref(y)) >= 0) {
    if (mpz_sgn(mpq_numref(y))*mpz_sgn(mpq_denref(y)) > 0)
      mpz_add_ui(x,x,1);
    else
      mpz_sub_ui(x,x,1);
  }
  mpz_clear(r);
}
void mpq_inner(mpq_t x, mpq_t *mtx1, mpq_t *mtx2, unsigned int len) {
  mpq_t tmp;
  unsigned int i;
  mpq_init(tmp);
  mpq_mul(x, *mtx1, *mtx2);
  for(i=1; i<len; i++) {
    mpq_mul(tmp,*(mtx1+i),*(mtx2+i));
    mpq_add(x,x,tmp);
  }
  mpq_clear(tmp);
}
void mpq_inner_z(mpq_t x, mpq_t *mtx1, mpz_t *mtx2, unsigned int len) {
  mpq_t tmp;
  unsigned int i;
  mpq_init(tmp);
  mpq_set_z(x, *mtx2);
  mpq_mul(x, x, *mtx1);
  for(i=1; i<len; i++) {
    mpq_set_z(tmp, *(mtx2+i));
    mpq_mul(tmp,tmp,*(mtx1+i));
    mpq_add(x,x,tmp);
  }
  mpq_clear(tmp);
}
