/*
 * Copyright (c) 2012-2015, Kosaku Nagasaka <nagasaka@main.h.kobe-u.ac.jp>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <stdlib.h>
#include <snap.h>

void mpz_inner(mpz_t x, mpz_t *mtx1, mpz_t *mtx2, unsigned int len) {
  unsigned int i;
  mpz_mul(x, *mtx1, *mtx2);
  for(i=1; i<len; i++) {
    mpz_addmul(x,*(mtx1+i),*(mtx2+i));
  }
}

void mpz_inner_matrix_transpose(mpz_t *mtx3, mpz_t *mtx1, mpz_t *mtx2, unsigned int rows1, unsigned int cols, unsigned int rows2) {
  mpz_t tmp;
  unsigned int i,j,k;
  mpz_init(tmp);
  for(i=0; i<rows1; i++) {
    for(j=i; j<rows2; j++) {
      mpz_set_ui(tmp,0);
      for(k=0; k<cols; k++) {
        mpz_addmul(tmp,*(mtx1+i*cols+k),*(mtx2+j*cols+k));
      }
      mpz_set(*(mtx3+i*rows2+j),tmp);
      mpz_set(*(mtx3+j*rows2+i),tmp);
    }
  }
  mpz_clear(tmp);
}

void mpz_rdiv_q(mpz_t x, mpz_t num, mpz_t den) {
  mpz_t r;
  mpz_init(r);
  mpz_tdiv_qr(x, r,num, den);
  mpz_mul_ui(r,r,2);
  if (mpz_cmpabs(r,den) >= 0) {
    if (mpz_sgn(num)*mpz_sgn(den) > 0)
      mpz_add_ui(x,x,1);
    else
      mpz_sub_ui(x,x,1);
  }
  mpz_clear(r);
}

void mpz_mul_fr(mpz_t rop, mpz_t op1, mpfr_t op2, mpfr_rnd_t mode) {
  mpfr_t ropfr;
  mpfr_init2(ropfr, mpz_sizeinbase(op1,2)+mpfr_get_prec(op2));
  mpfr_set_z(ropfr, op1, mode);
  mpfr_mul(ropfr, ropfr, op2, mode);
  mpfr_get_z(rop, ropfr, mode);
  mpfr_clear(ropfr);
}
void mpz_mul_d(mpz_t rop, mpz_t op1, double op2) {
  mpfr_t ropfr;
  mpfr_init2(ropfr,mpz_sizeinbase(op1,2)+53);
  mpfr_set_z(ropfr, op1, SNAPLIB_MPFR_RND);
  mpfr_mul_d(ropfr, ropfr, op2, SNAPLIB_MPFR_RND);
  mpfr_get_z(rop, ropfr, SNAPLIB_MPFR_RND);
  mpfr_clear(ropfr);
}
void mpz_div_fr(mpz_t rop, mpz_t op1, mpfr_t op2, mpfr_rnd_t mode) {
  mpfr_t ropfr;
  mpfr_init2(ropfr,mpz_sizeinbase(op1,2)+mpfr_get_prec(op2));
  mpfr_set_z(ropfr, op1, mode);
  mpfr_div(ropfr, ropfr, op2, mode);
  mpfr_get_z(rop, ropfr, mode);
  mpfr_clear(ropfr);
}
void mpz_div_d(mpz_t rop, mpz_t op1, double op2) {
  mpfr_t ropfr;
  mpfr_init2(ropfr,mpz_sizeinbase(op1,2)+53);
  mpfr_set_z(ropfr, op1, SNAPLIB_MPFR_RND);
  mpfr_div_d(ropfr, ropfr, op2, SNAPLIB_MPFR_RND);
  mpfr_get_z(rop, ropfr, SNAPLIB_MPFR_RND);
  mpfr_clear(ropfr);
}

