/*
 * Copyright (c) 2017, Kosaku Nagasaka <nagasaka@main.h.kobe-u.ac.jp>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <math.h>
#include <signal.h>
#include <readline/readline.h>
#include <readline/history.h>

#include <time.h>
#include <sys/time.h>
#include <sys/resource.h>

#include <cblas.h>
#include <clapack.h>
#include <lapacke.h>

#include "snap.h"


void show_sdmd(sdmd_t mtx) {
  unsigned int i,j;
  for(i=0; i<sdmd_rows(mtx); i++) {
    for(j=0; j<sdmd_cols(mtx); j++) {
      printf("%+.5le ",sdmd_elem(mtx,i,j));
    }
    printf("\n");
  }
}

void interrupted(int sig) {
  printf("Program Terminated due to Signal %d.\n", sig);
  printf("Please do not enter CTRL-D at the beginning of inputs.\n");
  exit(1);
}

int snap_lapack_dgelsy_givens(unsigned int rows, unsigned int cols, unsigned int nrhs, double *matA, unsigned int ldA, double *vecB, unsigned int ldB, lapack_int *jpvt, double relative_tolerance, lapack_int *rank);

int main() {
  sdmd_t matA, vecB;
  lapack_int *jpvt;
  lapack_int rank;

  jpvt = (lapack_int*) malloc(3*sizeof(lapack_int));
  sdmd_init2(matA, 3, 2);
  sdmd_init2(vecB, 1, 3);

  sdmd_elem(matA, 0, 0) = 1;
  sdmd_elem(matA, 0, 1) = 2;
  sdmd_elem(matA, 1, 0) = 3;
  sdmd_elem(matA, 1, 1) = 4;
  sdmd_elem(matA, 2, 0) = 5;
  sdmd_elem(matA, 2, 1) = 6;
  sdmd_elem(vecB, 0, 0) = 7;
  sdmd_elem(vecB, 0, 1) = 8;
  sdmd_elem(vecB, 0, 2) = 9;

  show_sdmd(matA);
  show_sdmd(vecB);

  snap_lapack_dgelsy_givens(3, 2, 1, sdmd_elems(matA), 2, sdmd_elems(vecB), 1, jpvt, 0.0, &rank);

  show_sdmd(matA);
  show_sdmd(vecB);

  free(jpvt);
  sdmd_clear(vecB);
  sdmd_clear(matA);
  return (0);
}
